require 'spec_helper'

describe RocketGate::Authorization do
  let(:authorization) { RocketGate::Authorization.new }

  describe '#avs_ok?' do
    subject { authorization.avs_ok? }

    context 'when AVS verification is required' do
      before { RocketGate.configuration.require_avs = true }

      it 'returns true when the AVS check was successful' do
        authorization.avs_response = :match
        expect(subject).to be true
      end

      it 'returns false when the AVS check was unsuccessful' do
        authorization.avs_response = :no_match
        expect(subject).to be false
      end
    end

    context 'when AVS verification is not required' do
      before { RocketGate.configuration.require_avs = false }

      it 'returns true' do
        expect(subject).to be true
      end
    end
  end

  describe '#cvv_ok?' do
    subject { authorization.cvv_ok? }

    context 'when CVV verification is required' do
      before { RocketGate.configuration.require_cvv = true }

      it 'returns true when the CVV check was successful' do
        authorization.cvv_response = :match
        expect(subject).to be true
      end

      it 'returns false when the CVV check was unsuccessful' do
        authorization.cvv_response = :no_match
        expect(subject).to be false
      end
    end

    context 'when CVV verification is not required' do
      before { RocketGate.configuration.require_cvv = false }

      it 'returns true' do
        expect(subject).to be true
      end
    end
  end

  describe '#declined?' do
    subject { authorization.declined? }

    it 'returns true when successful' do
      allow(authorization).to receive(:success?).and_return(true)
      expect(subject).to be false
    end

    it 'returns false when unsuccessful' do
      allow(authorization).to receive(:success?).and_return(false)
      expect(subject).to be true
    end
  end

  describe '#success?' do
    subject { authorization.success? }

    it 'returns true when everything is ok' do
      allow(authorization).to receive(:avs_ok?).and_return(true)
      allow(authorization).to receive(:cvv_ok?).and_return(true)
      allow(authorization).to receive(:reason_code).and_return(:success)
      expect(subject).to be true
    end

    it 'returns false when anything is not ok' do
      allow(authorization).to receive(:avs_ok?).and_return(false)
      expect(subject).to be false
    end
  end
end
