require 'spec_helper'

describe RocketGate::Connection do
  before { allow(Net::HTTP).to receive(:new).and_return(client) }
  let(:client) { Net::HTTP.new('insecure.com', 80) }

  describe '.new' do
    subject { RocketGate::Connection.new('example.com', 443, '/path') }

    it 'instantiates a client' do
      expect(Net::HTTP).to receive(:new).with('example.com', 443)
      subject
    end

    it 'sets the request timeouts' do
      expect(client).to receive(:open_timeout=).with(10)
      expect(client).to receive(:read_timeout=).with(10)
      subject
    end

    context 'when SSL is required' do
      before { RocketGate.configuration.require_ssl = true }

      it 'uses SSL, VERIFY_NONE mode' do
        expect(client).to receive(:use_ssl=).with(true)
        expect(client).to receive(:verify_mode=).with(OpenSSL::SSL::VERIFY_NONE)
        subject
      end
    end

    context 'when SSL is not required' do
      before { RocketGate.configuration.require_ssl = false }

      it 'uses SSL, VERIFY_NONE mode' do
        expect(client).not_to receive(:use_ssl=)
        expect(client).not_to receive(:verify_mode=)
        subject
      end
    end
  end

  describe '#make_request!' do
    subject { RocketGate::Connection.new('example.com', 443, '/path').make_request!('test') }
    let(:request) { double(:post).as_null_object }
    before do
      allow(Net::HTTP::Post).to receive(:new).and_return(request)
      allow(client).to receive(:request).with(request)
    end

    it 'builds a request' do
      expect(Net::HTTP::Post).to receive(:new).with('/path', {
        'Accept'       => 'text/xml',
        'Content-Type' => 'text/xml',
        'User-Agent'   => 'rocketgate-ruby/0.0.2'
      }).and_return(request)
      expect(request).to receive(:body=).with('test')
      subject
    end

    it 'sends the request' do
      expect(client).to receive(:request).with(request)
      subject
    end
  end
end
