require 'spec_helper'

describe RocketGate::CreditCard do
  describe '.new' do
    it 'accepts an arbitrary set of arguments and assigns them to certain attributes' do
      credit_card = RocketGate::CreditCard.new('4111111111111111', '01', '2015', '123')
      expect(credit_card.number).to eq '4111111111111111'
      expect(credit_card.exp_month).to eq '01'
      expect(credit_card.exp_year).to eq '2015'
      expect(credit_card.cvv).to eq '123'
    end

    it 'cleans the card number' do
      expect(RocketGate::CreditCard.new('4111-2222-3333-4444').number).to eq '4111222233334444'
      expect(RocketGate::CreditCard.new('4444 5555 6666 7777').number).to eq '4444555566667777'
    end
  end

  describe '.from_card_hash' do
    it 'instantiates a card with the hash' do
      credit_card = RocketGate::CreditCard.from_card_hash('m77xlHZiPKVsF9p1/VdzTb+CUwaGBDpuSRxtcb7+j24=')
      expect(credit_card.card_hash).to eq 'm77xlHZiPKVsF9p1/VdzTb+CUwaGBDpuSRxtcb7+j24='
    end
  end

  describe '#to_hash' do
    let(:credit_card) { RocketGate::CreditCard.new('4111111111111111', '01', '2015', '123') }

    context 'when there is a card hash available' do
      before { credit_card.card_hash = 'm77xlHZiPKVsF9p1/VdzTb+CUwaGBDpuSRxtcb7+j24=' }

      it 'returns a hash which only includes the card hash' do
        expect(credit_card.to_hash).to eq({ cardHash: 'm77xlHZiPKVsF9p1/VdzTb+CUwaGBDpuSRxtcb7+j24=' })
      end
    end

    context 'when there is no card hash available' do
      it 'returns a hash of the card attributes' do
        expect(credit_card.to_hash).to eq({
          cardNo: '4111111111111111',
          expireMonth: '01',
          expireYear: '2015',
          cvv2: '123'
        })
      end
    end
  end

  describe '#valid?' do
    context 'when there is a card hash' do
      let(:credit_card) { RocketGate::CreditCard.new.tap{|cc| cc.card_hash = 'abc' } }

      it 'is true' do
        expect(credit_card.valid?).to eq true
      end
    end

    context 'when there is not a card hash' do
      let(:credit_card) { RocketGate::CreditCard.new }

      it 'validates the other attributes' do
        expect(credit_card.valid?).to eq false

        credit_card.number = '4111111111111111',
        credit_card.exp_month = '01'
        credit_card.exp_year = '2015'
        credit_card.cvv = '123'

        expect(credit_card.valid?).to eq true
      end
    end
  end
end
