require 'spec_helper'

describe RocketGate::Customer do
  describe '.new' do
    it 'accepts an arbitrary set of arguments and assigns them to certain attributes' do
      customer = RocketGate::Customer.new('Test', 'Person', '123 Fake Street', 'Beverly Hills', 'CA', '90210', 'US', 'test@example.com', 'tester', '127.0.0.1', '07f7b231-9b29-46d4-9c19-cd6790e712b5')
      expect(customer.first_name).to eq 'Test'
      expect(customer.last_name).to eq 'Person'
      expect(customer.street_address).to eq '123 Fake Street'
      expect(customer.city).to eq 'Beverly Hills'
      expect(customer.state).to eq 'CA'
      expect(customer.postal_code).to eq '90210'
      expect(customer.country).to eq 'US'
      expect(customer.email).to eq 'test@example.com'
      expect(customer.username).to eq 'tester'
      expect(customer.ip_address).to eq '127.0.0.1'
      expect(customer.id).to eq '07f7b231-9b29-46d4-9c19-cd6790e712b5'
    end
  end

  describe '#id' do
    let(:customer) { RocketGate::Customer.new }

    it 'returns the supplied ID when set' do
      customer.id = 'abc123'
      expect(customer.id).to eq 'abc123'
    end

    it 'generates a random UUID when not set' do
      expect(SecureRandom).to receive(:uuid).and_return('07f7b231-9b29-46d4-9c19-cd6790e712b5')
      expect(customer.id).to eq '07f7b231-9b29-46d4-9c19-cd6790e712b5'
    end
  end

  describe '#to_hash' do
    let(:customer) { RocketGate::Customer.new('Test', 'Person', '123 Fake Street', 'Beverly Hills', 'CA', '90210', 'US', 'test@example.com', 'tester', '127.0.0.1', '07f7b231-9b29-46d4-9c19-cd6790e712b5') }
    let(:expected) do
      {
        merchantCustomerID: '07f7b231-9b29-46d4-9c19-cd6790e712b5',
        customerFirstName: 'Test',
        customerLastName: 'Person',
        billingAddress: '123 Fake Street',
        billingCity: 'Beverly Hills',
        billingState: 'CA',
        billingZipCode: '90210',
        billingCountry: 'US',
        email: 'test@example.com',
        username: 'tester',
        ipAddress: '127.0.0.1'
      }
    end

    it 'returns a hash of the customer attributes' do
      expect(customer.to_hash).to eq expected
    end
  end
end
