require 'spec_helper'

describe RocketGate::Validatable do
  describe '.validatable' do
    let(:test_class) { Class.new }

    subject do
      test_class.class_eval do
        include RocketGate::Validatable
        attr_accessor :one, :two, :three
        validatable :one, :two, :four
      end
    end

    it 'records a list of attributes to validate' do
      expect(test_class).to receive(:validatable).with(:one, :two, :four)
      subject
    end

    it 'complains about attributes that are not real' do
      expect{subject}.to raise_error(RocketGate::ValidationError, 'Invalid attribute: four')
    end
  end

  describe '#valid?' do
    let(:test_class) { Class.new }

    before do
      test_class.class_eval do
        include RocketGate::Validatable
        attr_accessor :one, :two
        validatable :one, :two
      end
    end

    let(:validatable) { test_class.new }

    it 'returns true if all attributes are set' do
      validatable.one = 1
      validatable.two = 2
      expect(validatable.valid?).to eq true
    end

    it 'returns false if an attribute is not set' do
      validatable.one = 1
      validatable.two = nil
      expect(validatable.valid?).to eq false
    end
  end

  describe '#validate!' do
    let(:test_class) { Class.new }

    before do
      test_class.class_eval do
        include RocketGate::Validatable
        attr_accessor :one
        validatable :one
      end
    end

    let(:validatable) { test_class.new }

    context 'when it is valid' do
      it 'returns itself' do
        validatable.one = 1
        expect(validatable.validate!).to eq validatable
      end
    end

    context 'when it is invalid' do
      it 'raises an error with the invalid attribute names' do
        validatable.one = nil
        message = 'Invalid attributes: one'
        expect{validatable.validate!}.to raise_exception(RocketGate::ValidationError, message)
      end
    end
  end
end
